@@ -48,11 +48,11 @@ appropriately.
 See https://github.com/tcalmant/jsonrpclib for more info.
 
 :license: Apache License 2.0
-:version: 0.2.1
+:version: 0.2.3
 """
 
 # Module version
-__version_info__ = (0, 2, 1)
+__version_info__ = (0, 2, 3)
 __version__ = ".".join(str(x) for x in __version_info__)
 
 # Documentation strings format
@@ -66,9 +66,13 @@ import jsonrpclib.utils as utils
 
 # Standard library
 import contextlib
+import logging
 import sys
 import uuid
 
+# Create the logger
+_logger = logging.getLogger(__name__)
+
 try:
     # Python 3
     # pylint: disable=F0401,E0611
@@ -99,6 +103,7 @@ try:
     # pylint: disable=F0401,E0611
     # Using cjson
     import cjson
+    _logger.debug("Using cjson as JSON library")
 
     # Declare cjson methods
     def jdumps(obj, encoding='utf-8'):
@@ -119,10 +124,14 @@ except ImportError:
     # Use json or simplejson
     try:
         import json
+        _logger.debug("Using json as JSON library")
+
     except ImportError:
         try:
             import simplejson as json
+            _logger.debug("Using simplejson as JSON library")
         except ImportError:
+            _logger.error("No supported JSON library found")
             raise ImportError('You must have the cjson, json, or simplejson '
                               'module(s) available.')
 
@@ -183,6 +192,67 @@ class AppError(ProtocolError):
         return self.args[0][2]
 
 
+class JSONParser(object):
+    """
+    Default JSON parser
+    """
+    def __init__(self, target):
+        """
+        Associates the target loader to the parser
+
+        :param target: a JSONTarget instance
+        """
+        self.target = target
+
+    def feed(self, data):
+        """
+        Feeds the associated target with the given data
+        """
+        self.target.feed(data)
+
+    def close(self):
+        """
+        Does nothing
+        """
+        pass
+
+
+class JSONTarget(object):
+    """
+    Unmarshalls stream data to a string
+    """
+    def __init__(self):
+        """
+        Sets up the unmarshaller
+        """
+        self.data = []
+
+    def feed(self, data):
+        """
+        Stores the given raw data into a buffer
+        """
+        # Store raw data as it might not contain whole wide-character
+        self.data.append(data)
+
+    def close(self):
+        """
+        Unmarshalls the buffered data
+        """
+        if not self.data:
+            return ''
+        else:
+            # Use type to have a valid join (str vs. bytes)
+            data = type(self.data[0])().join(self.data)
+            try:
+                # Convert the whole final string
+                data = utils.from_bytes(data)
+            except:
+                # Try a pass-through
+                pass
+
+            return data
+
+
 class TransportMixIn(object):
     """ Just extends the XMLRPC transport where necessary. """
     # for Python 2.7 support
@@ -195,7 +265,7 @@ class TransportMixIn(object):
     # Use the configuration to change the content-type
     readonly_headers = ('content-length', 'content-type')
 
-    def __init__(self, config=jsonrpclib.config.DEFAULT):
+    def __init__(self, config=jsonrpclib.config.DEFAULT, context=None):
         """
         Sets up the transport
 
@@ -204,6 +274,9 @@ class TransportMixIn(object):
         # Store the configuration
         self._config = config
 
+        # Store the SSL context
+        self.context = context
+
         # Set up the user agent
         self.user_agent = config.user_agent
 
@@ -284,67 +357,6 @@ class TransportMixIn(object):
         return JSONParser(target), target
 
 
-class JSONParser(object):
-    """
-    Default JSON parser
-    """
-    def __init__(self, target):
-        """
-        Associates the target loader to the parser
-
-        :param target: a JSONTarget instance
-        """
-        self.target = target
-
-    def feed(self, data):
-        """
-        Feeds the associated target with the given data
-        """
-        self.target.feed(data)
-
-    def close(self):
-        """
-        Does nothing
-        """
-        pass
-
-
-class JSONTarget(object):
-    """
-    Unmarshalls stream data to a string
-    """
-    def __init__(self):
-        """
-        Sets up the unmarshaller
-        """
-        self.data = []
-
-    def feed(self, data):
-        """
-        Stores the given raw data into a buffer
-        """
-        # Store raw data as it might not contain whole wide-character
-        self.data.append(data)
-
-    def close(self):
-        """
-        Unmarshalls the buffered data
-        """
-        if not self.data:
-            return ''
-        else:
-            # Use type to have a valid join (str vs. bytes)
-            data = type(self.data[0])().join(self.data)
-            try:
-                # Convert the whole final string
-                data = utils.from_bytes(data)
-            except:
-                # Try a pass-through
-                pass
-
-            return data
-
-
 class Transport(TransportMixIn, XMLTransport):
     """
     Mixed-in HTTP transport
@@ -368,7 +380,7 @@ class ServerProxy(XMLServerProxy):
     """
     def __init__(self, uri, transport=None, encoding=None,
                  verbose=0, version=None, headers=None, history=None,
-                 config=jsonrpclib.config.DEFAULT):
+                 config=jsonrpclib.config.DEFAULT, context=None):
         """
         Sets up the server proxy
 
@@ -380,6 +392,7 @@ class ServerProxy(XMLServerProxy):
         :param headers: Custom additional headers for each request
         :param history: History object (for tests)
         :param config: A JSONRPClib Config instance
+        :param context: The optional SSLContext to use
         """
         # Store the configuration
         self._config = config
@@ -387,6 +400,8 @@ class ServerProxy(XMLServerProxy):
 
         schema, uri = splittype(uri)
         if schema not in ('http', 'https'):
+            _logger.error("jsonrpclib only support http(s) URIs, not %s",
+                          schema)
             raise IOError('Unsupported JSON-RPC protocol.')
 
         self.__host, self.__handler = splithost(uri)
@@ -396,7 +411,7 @@ class ServerProxy(XMLServerProxy):
 
         if transport is None:
             if schema == 'https':
-                transport = SafeTransport(config=config)
+                transport = SafeTransport(config=config, context=context)
             else:
                 transport = Transport(config=config)
         self.__transport = transport
@@ -537,10 +552,10 @@ class _Method(XML_Method):
         """
         Sends an RPC request and returns the unmarshalled result
         """
-        if len(args) > 0 and len(kwargs) > 0:
+        if args and kwargs:
             raise ProtocolError("Cannot use both positional and keyword "
                                 "arguments (according to JSON-RPC spec.)")
-        if len(args) > 0:
+        if args:
             return self.__send(self.__name, args)
         else:
             return self.__send(self.__name, kwargs)
@@ -595,10 +610,10 @@ class MultiCallMethod(object):
         """
         Normalizes call parameters
         """
-        if len(kwargs) > 0 and len(args) > 0:
+        if kwargs and args:
             raise ProtocolError('JSON-RPC does not support both ' +
                                 'positional and keyword arguments.')
-        if len(kwargs) > 0:
+        if kwargs:
             self.params = kwargs
         else:
             self.params = args
@@ -1103,8 +1118,6 @@ def check_for_errors(result):
                 data = result['error'].get('data', None)
                 raise AppError((code, message, data))
 
-            raise ProtocolError((code, message))
-
         elif isinstance(result['error'], dict) and len(result['error']) == 1:
             # Error with a single entry ('reason', ...): use its content
             error_key = result['error'].keys()[0]
